@@ -43,6 +43,8 @@ MINIMAL_TASKS = [
     'libdlmalloc-debug',
     'libemmalloc',
     'libemmalloc-64bit',
+    'libgl',
+    'libhtml5',
     'libsockets',
     'libc_rt_wasm',
     'struct_info',
@@ -57,6 +59,7 @@ USER_TASKS = [
     'bzip2',
     'cocos2d',
     'freetype',
+    'giflib',
     'harfbuzz',
     'icu',
     'libjpeg',
@@ -98,7 +101,7 @@ Issuing 'embuilder.py build ALL' causes each task to be built.
 
 def build_port(port_name, lib_name):
   if force:
-    shared.Cache.erase_file(lib_name)
+    shared.Cache.erase_file(shared.Cache.get_lib_name(lib_name))
 
   system_libs.build_port(port_name, shared.Settings)
 
@@ -129,20 +132,14 @@ def main():
 
   if args.lto:
     shared.Settings.LTO = "full"
-    # Reconfigure the cache dir to reflect the change
-    shared.reconfigure_cache()
 
   if args.pic:
     shared.Settings.RELOCATABLE = 1
-    # Reconfigure the cache dir to reflect the change
-    shared.reconfigure_cache()
 
   if args.force:
     force = True
 
   # process tasks
-  libname = system_libs.Ports.get_lib_name
-
   auto_tasks = False
   tasks = args.targets
   if 'SYSTEM' in tasks:
@@ -158,16 +155,9 @@ def main():
     tasks = SYSTEM_TASKS + USER_TASKS
     auto_tasks = True
   if auto_tasks:
-    skip_tasks = []
-    if shared.Settings.RELOCATABLE:
-      # we don't support PIC + pthreads yet
-      for task in SYSTEM_TASKS + USER_TASKS:
-        if '-mt' in task:
-          skip_tasks.append(task)
-      print('Skipping building of %s, because we don\'t support threads and PIC code.' % ', '.join(skip_tasks))
     # cocos2d: must be ported, errors on
     # "Cannot recognize the target platform; are you targeting an unsupported platform?"
-    skip_tasks += ['cocos2d']
+    skip_tasks = ['cocos2d']
     tasks = [x for x in tasks if x not in skip_tasks]
     print('Building targets: %s' % ' '.join(tasks))
   for what in tasks:
@@ -177,12 +167,16 @@ def main():
       if force:
         library.erase()
       library.get_path()
+    elif what == 'sysroot':
+      if force:
+        shared.Cache.erase_file('sysroot_install.stamp')
+      system_libs.ensure_sysroot()
     elif what == 'struct_info':
       if force:
         shared.Cache.erase_file('generated_struct_info.json')
       emscripten.generate_struct_info()
     elif what == 'icu':
-      build_port('icu', libname('libicuuc'))
+      build_port('icu', 'libicuuc.a')
     elif what == 'zlib':
       shared.Settings.USE_ZLIB = 1
       build_port('zlib', 'libz.a')
@@ -190,49 +184,51 @@ def main():
     elif what == 'bzip2':
       build_port('bzip2', 'libbz2.a')
     elif what == 'bullet':
-      build_port('bullet', libname('libbullet'))
+      build_port('bullet', 'libbullet.a')
     elif what == 'vorbis':
-      build_port('vorbis', libname('libvorbis'))
+      build_port('vorbis', 'libvorbis.a')
     elif what == 'ogg':
-      build_port('ogg', libname('libogg'))
+      build_port('ogg', 'libogg.a')
+    elif what == 'giflib':
+      build_port('giflib', 'libgif.a')
     elif what == 'libjpeg':
-      build_port('libjpeg', libname('libjpeg'))
+      build_port('libjpeg', 'libjpeg.a')
     elif what == 'libpng':
-      build_port('libpng', libname('libpng'))
+      build_port('libpng', 'libpng.a')
     elif what == 'sdl2':
-      build_port('sdl2', libname('libSDL2'))
+      build_port('sdl2', 'libSDL2.a')
     elif what == 'sdl2-mt':
       shared.Settings.USE_PTHREADS = 1
-      build_port('sdl2', libname('libSDL2-mt'))
+      build_port('sdl2', 'libSDL2-mt.a')
       shared.Settings.USE_PTHREADS = 0
     elif what == 'sdl2-gfx':
-      build_port('sdl2_gfx', libname('libSDL2_gfx'))
+      build_port('sdl2_gfx', 'libSDL2_gfx.a')
     elif what == 'sdl2-image':
-      build_port('sdl2_image', libname('libSDL2_image'))
+      build_port('sdl2_image', 'libSDL2_image.a')
     elif what == 'sdl2-image-png':
       shared.Settings.SDL2_IMAGE_FORMATS = ["png"]
-      build_port('sdl2_image', libname('libSDL2_image_png'))
+      build_port('sdl2_image', 'libSDL2_image_png.a')
       shared.Settings.SDL2_IMAGE_FORMATS = []
     elif what == 'sdl2-image-jpg':
       shared.Settings.SDL2_IMAGE_FORMATS = ["jpg"]
-      build_port('sdl2_image', libname('libSDL2_image_jpg'))
+      build_port('sdl2_image', 'libSDL2_image_jpg.a')
       shared.Settings.SDL2_IMAGE_FORMATS = []
     elif what == 'sdl2-net':
-      build_port('sdl2_net', libname('libSDL2_net'))
+      build_port('sdl2_net', 'libSDL2_net.a')
     elif what == 'sdl2-mixer':
       old_formats = shared.Settings.SDL2_MIXER_FORMATS
       shared.Settings.SDL2_MIXER_FORMATS = []
-      build_port('sdl2_mixer', libname('libSDL2_mixer'))
+      build_port('sdl2_mixer', 'libSDL2_mixer.a')
       shared.Settings.SDL2_MIXER_FORMATS = old_formats
     elif what == 'sdl2-mixer-ogg':
       old_formats = shared.Settings.SDL2_MIXER_FORMATS
       shared.Settings.SDL2_MIXER_FORMATS = ["ogg"]
-      build_port('sdl2_mixer', libname('libSDL2_mixer_ogg'))
+      build_port('sdl2_mixer', 'libSDL2_mixer_ogg.a')
       shared.Settings.SDL2_MIXER_FORMATS = old_formats
     elif what == 'sdl2-mixer-mp3':
       old_formats = shared.Settings.SDL2_MIXER_FORMATS
       shared.Settings.SDL2_MIXER_FORMATS = ["mp3"]
-      build_port('sdl2_mixer', libname('libSDL2_mixer_mp3'))
+      build_port('sdl2_mixer', 'libSDL2_mixer_mp3.a')
       shared.Settings.SDL2_MIXER_FORMATS = old_formats
     elif what == 'freetype':
       build_port('freetype', 'libfreetype.a')
@@ -243,17 +239,19 @@ def main():
       build_port('harfbuzz', 'libharfbuzz-mt.a')
       shared.Settings.USE_PTHREADS = 0
     elif what == 'sdl2-ttf':
-      build_port('sdl2_ttf', libname('libSDL2_ttf'))
+      build_port('sdl2_ttf', 'libSDL2_ttf.a')
     elif what == 'cocos2d':
-      build_port('cocos2d', libname('libcocos2d'))
+      build_port('cocos2d', 'libcocos2d.a')
     elif what == 'regal':
-      build_port('regal', libname('libregal'))
+      build_port('regal', 'libregal.a')
     elif what == 'regal-mt':
       shared.Settings.USE_PTHREADS = 1
-      build_port('regal', libname('libregal-mt'))
+      build_port('regal', 'libregal-mt.a')
       shared.Settings.USE_PTHREADS = 0
     elif what == 'boost_headers':
-      build_port('boost_headers', libname('libboost_headers'))
+      build_port('boost_headers', 'libboost_headers.a')
+    elif what == 'mpg123':
+      build_port('mpg123', 'libmpg123.a')
     else:
       logger.error('unfamiliar build target: ' + what)
       return 1
