@@ -5,8 +5,7 @@
  * found in the LICENSE file.
  */
 
-#ifndef __emscripten_fetch_h__
-#define __emscripten_fetch_h__
+#pragma once
 
 #include <limits.h>
 #include <stddef.h>
@@ -23,18 +22,23 @@ extern "C" {
 
 // If passed, the intermediate streamed bytes will be passed in to the onprogress() handler. If not specified, the
 // onprogress() handler will still be called, but without data bytes.
+// Note: Firefox only as it depends on 'moz-chunked-arraybuffer'.
 #define EMSCRIPTEN_FETCH_STREAM_DATA 2
 
 // If passed, the final download will be stored in IndexedDB. If not specified, the file will only reside in browser memory.
 #define EMSCRIPTEN_FETCH_PERSIST_FILE 4
 
-// If the file already exists in IndexedDB, it is returned without redownload. If a partial transfer exists in IndexedDB,
-// the download will resume from where it left off and run to completion.
+// Looks up if the file already exists in IndexedDB, and if so, it is returned without redownload. If a partial transfer
+// exists in IndexedDB, the download will resume from where it left off and run to completion.
 // EMSCRIPTEN_FETCH_APPEND, EMSCRIPTEN_FETCH_REPLACE and EMSCRIPTEN_FETCH_NO_DOWNLOAD are mutually exclusive.
+// If none of these three flags is specified, the fetch operation is implicitly treated as if EMSCRIPTEN_FETCH_APPEND
+// had been passed.
 #define EMSCRIPTEN_FETCH_APPEND 8
 
 // If the file already exists in IndexedDB, the old file will be deleted and a new download is started.
 // EMSCRIPTEN_FETCH_APPEND, EMSCRIPTEN_FETCH_REPLACE and EMSCRIPTEN_FETCH_NO_DOWNLOAD are mutually exclusive.
+// If you would like to perform an XHR that neither reads or writes to IndexedDB, pass this flag EMSCRIPTEN_FETCH_REPLACE,
+// and do not pass the flag EMSCRIPTEN_FETCH_PERSIST_FILE.
 #define EMSCRIPTEN_FETCH_REPLACE 16
 
 // If specified, the file will only be looked up in IndexedDB, but if it does not exist, it is not attempted to be downloaded
@@ -47,7 +51,7 @@ extern "C" {
 #define EMSCRIPTEN_FETCH_SYNCHRONOUS 64
 
 // If specified, it will be possible to call emscripten_fetch_wait() on the fetch
-// to test or wair for its completion.
+// to test or wait for its completion.
 #define EMSCRIPTEN_FETCH_WAITABLE 128
 
 struct emscripten_fetch_t;
@@ -64,6 +68,7 @@ typedef struct emscripten_fetch_attr_t
 	void (*onsuccess)(struct emscripten_fetch_t *fetch);
 	void (*onerror)(struct emscripten_fetch_t *fetch);
 	void (*onprogress)(struct emscripten_fetch_t *fetch);
+	void (*onreadystatechange)(struct emscripten_fetch_t *fetch);
 
 	// EMSCRIPTEN_FETCH_* attributes
 	uint32_t attributes;
@@ -181,6 +186,24 @@ EMSCRIPTEN_RESULT emscripten_fetch_wait(emscripten_fetch_t *fetch, double timeou
 // onerror() handler will be called in the calling thread before this function returns.
 EMSCRIPTEN_RESULT emscripten_fetch_close(emscripten_fetch_t *fetch);
 
+// Gets the size (in bytes) of the response headers as plain text.
+// This must be called on the same thread as the fetch originated on.
+// Note that this will return 0 if readyState < HEADERS_RECEIVED.
+size_t emscripten_fetch_get_response_headers_length(emscripten_fetch_t *fetch);
+
+// Gets the response headers as plain text. dstSizeBytes should be headers_length + 1 (for the null terminator).
+// This must be called on the same thread as the fetch originated on.
+size_t emscripten_fetch_get_response_headers(emscripten_fetch_t *fetch, char *dst, size_t dstSizeBytes);
+
+// Converts the plain text headers into an array of strings. This array takes the form
+// {"key1", "value1", "key2", "value2", "key3", "value3", ..., 0 }; Note especially that the array
+// is terminated with a null pointer.
+char **emscripten_fetch_unpack_response_headers(const char *headersString);
+
+// This frees the memory used by the array of headers. Call this when finished with the data returned
+// by emscripten_fetch_unpack_response_headers.
+void emscripten_fetch_free_unpacked_response_headers(char **unpackedHeaders);
+
 #define emscripten_asmfs_open_t int
 
 // The following flags specify how opening files for reading works (from strictest behavior to most flexible)
@@ -229,6 +252,3 @@ uint64_t emscripten_asmfs_compute_memory_usage();
 #ifdef __cplusplus
 }
 #endif
-
-// ~__emscripten_fetch_h__
-#endif
